@@ -795,10 +795,16 @@ Bool compute_PCMPxSTRx ( /*OUT*/V128* resV,
       even if they would probably work.  Life is too short to have
       unvalidated cases in the code base. */
    switch (imm8) {
-      case 0x00: case 0x02: case 0x08: case 0x0A: case 0x0C: case 0x0E:
-      case 0x12: case 0x14: case 0x18: case 0x1A:
-      case 0x30: case 0x34: case 0x38: case 0x3A:
-      case 0x40: case 0x42: case 0x44: case 0x46: case 0x4A:
+      case 0x00: case 0x02:
+      case 0x08: case 0x0A: case 0x0C: case 0x0E:
+                 case 0x12: case 0x14:
+      case 0x18: case 0x1A:
+      case 0x30:            case 0x34:
+      case 0x38: case 0x3A:
+      case 0x40: case 0x42: case 0x44: case 0x46:
+                 case 0x4A:
+                 case 0x62:
+      case 0x70: case 0x72:
          break;
       default:
          return False;
@@ -1047,7 +1053,7 @@ Bool compute_PCMPxSTRx_wide ( /*OUT*/V128* resV,
       unvalidated cases in the code base. */
    switch (imm8) {
       case 0x01: case 0x03: case 0x09: case 0x0B: case 0x0D:
-      case 0x13:            case 0x1B:
+                 case 0x13: case 0x19: case 0x1B:
                             case 0x39: case 0x3B:
                  case 0x45:            case 0x4B:
          break;
