@@ -98,10 +98,11 @@ void ML_(ppSym) ( Int idx, const DiSym* sym )
    vg_assert(sym->pri_name);
    if (sec_names)
       vg_assert(sec_names);
-   VG_(printf)( "%5d:  %c%c %#8lx .. %#8lx (%u)      %s%s",
+   VG_(printf)( "%5d:  %c%c%c %#8lx .. %#8lx (%u)      %s%s",
                 idx,
                 sym->isText ? 'T' : '-',
                 sym->isIFunc ? 'I' : '-',
+                sym->isGlobal ? 'G' : '-',
                 sym->avmas.main, 
                 sym->avmas.main + sym->size - 1, sym->size,
                 sym->pri_name, sec_names ? " " : "" );
@@ -232,14 +233,9 @@ void ML_(ppDiCfSI) ( const XArray* /* of CfiExpr */ exprs,
 /*--- Adding stuff                                         ---*/
 /*------------------------------------------------------------*/
 
-/* Add a str to the string table, including terminating zero, and
-   return pointer to the string in vg_strtab.  Unless it's been seen
-   recently, in which case we find the old pointer and return that.
-   This avoids the most egregious duplications.
-
-   JSGF: changed from returning an index to a pointer, and changed to
-   a chunking memory allocator rather than reallocating, so the
-   pointers are stable.
+/* If not yet in strpool, add a str to the string pool including terminating
+   zero.
+   Return the pointer to the string in strpool.
 */
 const HChar* ML_(addStr) ( DebugInfo* di, const HChar* str, Int len )
 {
@@ -411,6 +407,18 @@ static inline void set_fndn_ix (struct _DebugInfo* di, Word locno, UInt fndn_ix)
    }
 }
 
+
+// Comment the below line to trace LOCTAB merging/canonicalising
+#define TRACE_LOCTAB_CANON(msg,prev_loc,cur_loc)
+#ifndef TRACE_LOCTAB_CANON
+#define TRACE_LOCTAB_CANON(msg,prev_loc,cur_loc)                        \
+   VG_(printf)("%s previous: addr %#lx, size %d, line %d, "             \
+               " current: addr %#lx, size %d, line %d.\n",              \
+               msg,                                                     \
+               (prev_loc)->addr, (prev_loc)->size, (prev_loc)->lineno,  \
+               (cur_loc)->addr, (cur_loc)->size, (cur_loc)->lineno);
+#endif
+
 /* Add a location to the location table. 
 */
 static void addLoc ( struct _DebugInfo* di, DiLoc* loc, UInt fndn_ix )
@@ -418,6 +426,22 @@ static void addLoc ( struct _DebugInfo* di, DiLoc* loc, UInt fndn_ix )
    /* Zero-sized locs should have been ignored earlier */
    vg_assert(loc->size > 0);
 
+   /* Check if the last entry has adjacent range for the same line. */
+   if (di->loctab_used > 0) {
+      DiLoc *previous = &di->loctab[di->loctab_used - 1];
+      if ((previous->lineno == loc->lineno)
+          && (previous->addr + previous->size == loc->addr)) {
+         if (previous->size + loc->size <= MAX_LOC_SIZE) {
+            TRACE_LOCTAB_CANON ("addLoc merging", previous, loc);
+            previous->size += loc->size;
+            return;
+         } else {
+            TRACE_LOCTAB_CANON ("addLoc merging not done (maxsize)",
+                                previous, loc);
+         }
+      }
+   }
+
    if (di->loctab_used == di->loctab_size) {
       UInt   new_sz;
       DiLoc* new_loctab;
@@ -464,6 +488,20 @@ static void shrinkLocTab ( struct _DebugInfo* di )
    di->loctab_size = new_sz;
 }
 
+#define COMPLAIN_ONCE(what, limit, limit_op)                   \
+   {                                                           \
+   static Bool complained = False;                             \
+   if (!complained) {                                          \
+      complained = True;                                       \
+      VG_(message)(Vg_UserMsg,                                 \
+                   "warning: Can't handle " what " with "      \
+                   "line number %d " limit_op " than %d\n",    \
+                   lineno, limit);                             \
+      VG_(message)(Vg_UserMsg,                                 \
+                   "(Nb: this message is only shown once)\n"); \
+   } \
+}
+
 
 /* Top-level place to call to add a source-location mapping entry.
 */
@@ -538,30 +576,11 @@ void ML_(addLineInfo) ( struct _DebugInfo* di,
    }
 
    if (lineno < 0) {
-      static Bool complained = False;
-      if (!complained) {
-         complained = True;
-         VG_(message)(Vg_UserMsg, 
-                      "warning: ignoring line info entry with "
-                      "negative line number (%d)\n", lineno);
-         VG_(message)(Vg_UserMsg, 
-                      "(Nb: this message is only shown once)\n");
-      }
+      COMPLAIN_ONCE("line info entry", 0, "smaller");
       return;
    }
    if (lineno > MAX_LINENO) {
-      static Bool complained = False;
-      if (!complained) {
-         complained = True;
-         VG_(message)(Vg_UserMsg, 
-                      "warning: ignoring line info entry with "
-                      "huge line number (%d)\n", lineno);
-         VG_(message)(Vg_UserMsg, 
-                      "         Can't handle line numbers "
-                      "greater than %d, sorry\n", MAX_LINENO);
-         VG_(message)(Vg_UserMsg, 
-                      "(Nb: this message is only shown once)\n");
-      }
+      COMPLAIN_ONCE("line info entry", MAX_LINENO, "greater");
       return;
    }
 
@@ -639,20 +658,12 @@ void ML_(addInlInfo) ( struct _DebugInfo* di,
        addr_hi = addr_lo + 1;
    }
 
-   vg_assert(lineno >= 0);
+   if (lineno < 0) {
+      COMPLAIN_ONCE ("inlined call info entry", 0, "smaller");
+      return;
+   }
    if (lineno > MAX_LINENO) {
-      static Bool complained = False;
-      if (!complained) {
-         complained = True;
-         VG_(message)(Vg_UserMsg, 
-                      "warning: ignoring inlined call info entry with "
-                      "huge line number (%d)\n", lineno);
-         VG_(message)(Vg_UserMsg, 
-                      "         Can't handle line numbers "
-                      "greater than %d, sorry\n", MAX_LINENO);
-         VG_(message)(Vg_UserMsg, 
-                      "(Nb: this message is only shown once)\n");
-      }
+      COMPLAIN_ONCE ("inlined call info entry", MAX_LINENO, "greater");
       return;
    }
 
@@ -1646,7 +1657,7 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
    Word  i, j, n_truncated;
    Addr  sta1, sta2, end1, end2, toc1, toc2;
    const HChar *pri1, *pri2, **sec1, **sec2;
-   Bool  ist1, ist2, isf1, isf2;
+   Bool  ist1, ist2, isf1, isf2, isg1, isg2;
 
 #  define SWAP(ty,aa,bb) \
       do { ty tt = (aa); (aa) = (bb); (bb) = tt; } while (0)
@@ -1693,6 +1704,8 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
             }
             /* mark w as an IFunc if either w or r are */
             di->symtab[w].isIFunc = di->symtab[w].isIFunc || di->symtab[r].isIFunc;
+            /* likewise for global symbols */
+            di->symtab[w].isGlobal = di->symtab[w].isGlobal || di->symtab[r].isGlobal;
             /* and use ::pri_names to indicate this slot is no longer in use */
             di->symtab[r].pri_name = NULL;
             if (di->symtab[r].sec_names) {
@@ -1796,6 +1809,7 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
       sec1 = di->symtab[i].sec_names;
       ist1 = di->symtab[i].isText;
       isf1 = di->symtab[i].isIFunc;
+      isg1 = di->symtab[i].isGlobal;
 
       sta2 = di->symtab[i+1].avmas.main;
       end2 = sta2 + di->symtab[i+1].size - 1;
@@ -1805,6 +1819,7 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
       sec2 = di->symtab[i+1].sec_names;
       ist2 = di->symtab[i+1].isText;
       isf2 = di->symtab[i+1].isIFunc;
+      isg2 = di->symtab[i+1].isGlobal;
 
       if (sta1 < sta2) {
          end1 = sta2 - 1;
@@ -1814,7 +1829,7 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
             sta1 = end2 + 1;
             SWAP(Addr,sta1,sta2); SWAP(Addr,end1,end2); SWAP(Addr,toc1,toc2);
             SWAP(const HChar*,pri1,pri2); SWAP(const HChar**,sec1,sec2);
-            SWAP(Bool,ist1,ist2); SWAP(Bool,isf1,isf2);
+            SWAP(Bool,ist1,ist2); SWAP(Bool,isf1,isf2); SWAP(Bool, isg1, isg2);
          } else 
          if (end1 < end2) {
             sta2 = end1 + 1;
@@ -1831,6 +1846,7 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
       di->symtab[i].sec_names = sec1;
       di->symtab[i].isText    = ist1;
       di->symtab[i].isIFunc   = isf1;
+      di->symtab[i].isGlobal  = isg1;
 
       di->symtab[i+1].avmas.main = sta2;
       di->symtab[i+1].size       = end2 - sta2 + 1;
@@ -1840,6 +1856,7 @@ static void canonicaliseSymtab ( struct _DebugInfo* di )
       di->symtab[i+1].sec_names = sec2;
       di->symtab[i+1].isText    = ist2;
       di->symtab[i+1].isIFunc   = isf2;
+      di->symtab[i+1].isGlobal  = isg2;
 
       vg_assert(sta1 <= sta2);
       vg_assert(di->symtab[i].size > 0);
@@ -1994,13 +2011,15 @@ static void canonicaliseLoctab ( struct _DebugInfo* di )
    /* sort loctab and loctab_fndn_ix by addr. */
    sort_loctab_and_loctab_fndn_ix (di);
 
-   /* If two adjacent entries overlap, truncate the first. */
    for (i = 0; i < ((Word)di->loctab_used)-1; i++) {
       vg_assert(di->loctab[i].size < 10000);
+      /* If two adjacent entries overlap, truncate the first. */
       if (di->loctab[i].addr + di->loctab[i].size > di->loctab[i+1].addr) {
          /* Do this in signed int32 because the actual .size fields
             are only 12 bits. */
          Int new_size = di->loctab[i+1].addr - di->loctab[i].addr;
+         TRACE_LOCTAB_CANON ("Truncating",
+                             &(di->loctab[i]), &(di->loctab[i+1]));
          if (new_size < 0) {
             di->loctab[i].size = 0;
          } else
