@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2005-2013 Apple Inc.
+   Copyright (C) 2005-2015 Apple Inc.
       Greg Parker  gparker@apple.com
 
    This program is free software; you can redistribute it and/or
@@ -51,7 +51,7 @@ extern const UInt ML_(mdep_trap_table_size);
 
 void VG_(show_open_ports)(void);
 
-Bool ML_(sync_mappings)(const HChar *when, const HChar *where, Int num);
+Bool ML_(sync_mappings)(const HChar *when, const HChar *where, UWord num);
 
 // Unix syscalls.  
 // GEN = it uses the generic wrapper
@@ -189,7 +189,7 @@ DECL_TEMPLATE(darwin, getsockopt);              // 118
 // old truncate
 // old ftruncate
 // GEN flock 131
-// NYI mkfifo 132
+DECL_TEMPLATE(darwin, mkfifo);                  // 132
 DECL_TEMPLATE(darwin, sendto);                  // 133
 DECL_TEMPLATE(darwin, shutdown);                // 134
 DECL_TEMPLATE(darwin, socketpair);              // 135
@@ -332,7 +332,13 @@ DECL_TEMPLATE(darwin, sem_unlink);              // 270
 DECL_TEMPLATE(darwin, sem_wait);                // 271
 DECL_TEMPLATE(darwin, sem_trywait);             // 272
 DECL_TEMPLATE(darwin, sem_post);                // 273
+
+#if DARWIN_VERS < DARWIN_10_10
 // NYI sem_getvalue 274
+#elif DARWIN_VERS >= DARWIN_10_10
+DECL_TEMPLATE(darwin, sysctlbyname);            // 274
+#endif
+
 DECL_TEMPLATE(darwin, sem_init);                // 275
 DECL_TEMPLATE(darwin, sem_destroy);             // 276
 DECL_TEMPLATE(darwin, open_extended)            // 277
@@ -344,7 +350,7 @@ DECL_TEMPLATE(darwin, chmod_extended);          // 282
 DECL_TEMPLATE(darwin, fchmod_extended);         // 283
 DECL_TEMPLATE(darwin, access_extended);         // 284
 DECL_TEMPLATE(darwin, settid);                  // 285
-#if DARWIN_VERS >= DARWIN_10_7
+#if DARWIN_VERS >= DARWIN_10_6
 DECL_TEMPLATE(darwin, gettid);                  // 286
 #endif
 // NYI setsgroups 287
@@ -356,23 +362,44 @@ DECL_TEMPLATE(darwin, gettid);                  // 286
 // NYI identitysvc 293
 // NYI shared_region_check_np 294
 // NYI shared_region_map_np 295
+#if DARWIN_VERS >= DARWIN_10_6
+// NYI vm_pressure_monitor 296
+// NYI psynch_rw_longrdlock 297
+// NYI psynch_rw_yieldwrlock 298
+// NYI psynch_rw_downgrade 299
+// NYI psynch_rw_upgrade 300
+DECL_TEMPLATE(darwin, psynch_mutexwait);       // 301
+DECL_TEMPLATE(darwin, psynch_mutexdrop);       // 302
+DECL_TEMPLATE(darwin, psynch_cvbroad);         // 303
+DECL_TEMPLATE(darwin, psynch_cvsignal);        // 304
+DECL_TEMPLATE(darwin, psynch_cvwait);          // 305
+DECL_TEMPLATE(darwin, psynch_rw_rdlock);       // 306
+DECL_TEMPLATE(darwin, psynch_rw_wrlock);       // 307
+DECL_TEMPLATE(darwin, psynch_rw_unlock);       // 308
+// NYI psynch_rw_unlock2 309
+#else
 // old load_shared_file
 // old reset_shared_file
 // old new_system_shared_regions
 // old shared_region_map_file_np
 // old shared_region_make_private_np
-DECL_TEMPLATE(darwin, psynch_mutexwait);       // 301 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_mutexdrop);       // 302 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_cvbroad);         // 303 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_cvsignal);        // 304 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_cvwait);          // 305 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_rw_rdlock);       // 306 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_rw_wrlock);       // 307 // new in 10.7 ?
-DECL_TEMPLATE(darwin, psynch_rw_unlock);       // 308 // new in 10.7 ?
+// NYI __pthread_mutex_destroy 301
+// NYI __pthread_mutex_init 302
+// NYI __pthread_mutex_lock 303
+// NYI __pthread_mutex_trylock 304
+// NYI __pthread_mutex_unlock 305
+// NYI __pthread_cond_init 306
+// NYI __pthread_cond_destroy 307
+// NYI __pthread_cond_broadcast 308
 // NYI __pthread_cond_signal 309
+#endif
 // NYI getsid 310
 // NYI settid_with_pid 311
-DECL_TEMPLATE(darwin, psynch_cvclrprepost);    // 312 // new in 10.7 ?
+#if DARWIN_VERS >= DARWIN_10_7
+DECL_TEMPLATE(darwin, psynch_cvclrprepost);    // 312
+#else
+// NYI __pthread_cond_timedwait 312
+#endif
 // NYI aio_fsync 313
 DECL_TEMPLATE(darwin, aio_return);             // 314
 DECL_TEMPLATE(darwin, aio_suspend);            // 315
@@ -383,7 +410,7 @@ DECL_TEMPLATE(darwin, aio_write);              // 319
 // NYI lio_listio 320
 // NYI __pthread_cond_wait 321
 // NYI iopolicysys 322
-// 323
+// NYI process_policy 323
 // NYI mlockall 324
 // NYI munlockall 325
 // 326
@@ -396,9 +423,7 @@ DECL_TEMPLATE(darwin, __pthread_markcancel);    // 332
 DECL_TEMPLATE(darwin, __pthread_canceled);      // 333
 DECL_TEMPLATE(darwin, __semwait_signal);        // 334
 // old utrace
-#if DARWIN_VERS >= DARWIN_10_6
 DECL_TEMPLATE(darwin, proc_info);               // 336
-#endif
 DECL_TEMPLATE(darwin, sendfile);                // 337
 DECL_TEMPLATE(darwin, stat64);                  // 338
 DECL_TEMPLATE(darwin, fstat64);                 // 339
@@ -410,8 +435,8 @@ DECL_TEMPLATE(darwin, getdirentries64);         // 344
 DECL_TEMPLATE(darwin, statfs64);                // 345
 DECL_TEMPLATE(darwin, fstatfs64);               // 346
 DECL_TEMPLATE(darwin, getfsstat64);             // 347
-// NYI __pthread_chdir 348
-// NYI __pthread_fchdir 349
+DECL_TEMPLATE(darwin, __pthread_chdir);         // 348
+DECL_TEMPLATE(darwin, __pthread_fchdir);        // 349
 // NYI audit 350
 DECL_TEMPLATE(darwin, auditon);                 // 351
 // 352
@@ -419,9 +444,7 @@ DECL_TEMPLATE(darwin, auditon);                 // 351
 // NYI setauid 354
 // NYI getaudit 355
 // NYI setaudit 356
-#if DARWIN_VERS >= DARWIN_10_7
 DECL_TEMPLATE(darwin, getaudit_addr)            // 357
-#endif
 // NYI setaudit_addr 358
 // NYI auditctl 359
 DECL_TEMPLATE(darwin, bsdthread_create);        // 360
@@ -436,7 +459,9 @@ DECL_TEMPLATE(darwin, workq_ops);               // 368
 DECL_TEMPLATE(darwin, kevent64);                // 369
 // 370
 // 371
+#if DARWIN_VERS >= DARWIN_10_6
 DECL_TEMPLATE(darwin, __thread_selfid);         // 372
+#endif
 // 373
 // 374
 // 375
@@ -496,6 +521,15 @@ DECL_TEMPLATE(darwin, audit_session_self);       // 428
 // NYI audit_session_join 429
 DECL_TEMPLATE(darwin, fileport_makeport);        // 430
 
+// NYI fileport_makefd 431
+// NYI audit_session_port 432
+// NYI pid_suspend 433
+// NYI pid_resume 434
+#if DARWIN_VERS >= DARWIN_10_10
+// NYI pid_hibernate 435
+// NYI pid_shutdown_sockets 436
+#endif /* DARWIN_VERS >= DARWIN_10_10 */
+// old old shared_region_slide_np 437
 // NYI shared_region_map_and_slide_np            // 438
 // NYI kas_info                                  // 439
 // NYI memorystatus_control                      // 440
@@ -503,10 +537,53 @@ DECL_TEMPLATE(darwin, guarded_open_np);          // 441
 DECL_TEMPLATE(darwin, guarded_close_np);         // 442
 DECL_TEMPLATE(darwin, guarded_kqueue_np);        // 443
 DECL_TEMPLATE(darwin, change_fdguard_np);        // 444
-
+// old __proc_suppress 445
 // NYI proc_rlimit_control                       // 446
 DECL_TEMPLATE(darwin, connectx);                 // 447
 DECL_TEMPLATE(darwin, disconnectx);              // 448
+// NYI peeloff // 449
+// NYI socket_delegate // 450
+// NYI telemetry // 451
+// NYI proc_uuid_policy // 452
+// NYI memorystatus_get_level // 453
+// NYI system_override // 454
+// NYI vfs_purge // 455
+#if DARWIN_VERS >= DARWIN_10_10
+// NYI sfi_ctl         // 456
+// NYI sfi_pidctl      // 457
+// NYI coalition       // 458
+// NYI coalition_info  // 459
+DECL_TEMPLATE(darwin, necp_match_policy);        // 460
+DECL_TEMPLATE(darwin, getattrlistbulk);          // 461
+// 462
+// NYI openat          // 463
+// NYI openat_nocancel // 464
+// NYI renameat        // 465
+// NYI faccessat       // 466
+// NYI fchmodat        // 467
+// NYI fchownat        // 468
+// NYI fstatat         // 469
+// NYI fstatat64       // 470
+// NYI linkat          // 471
+// NYI unlinkat        // 472
+DECL_TEMPLATE(darwin, readlinkat);               // 473
+// NYI symlinkat       // 474
+// NYI mkdirat         // 475
+// NYI getattrlistat   // 476
+// NYI proc_trace_log  // 477
+DECL_TEMPLATE(darwin, bsdthread_ctl);            // 478
+// NYI openbyid_np     // 479
+// NYI recvmsg_x       // 480
+// NYI sendmsg_x       // 481
+// NYI thread_selfusage  // 482
+// NYI csrctl          // 483
+DECL_TEMPLATE(darwin, guarded_open_dprotected_np);  // 484
+DECL_TEMPLATE(darwin, guarded_write_np);            // 485
+DECL_TEMPLATE(darwin, guarded_pwrite_np);           // 486
+DECL_TEMPLATE(darwin, guarded_writev_np);           // 487
+// NYI rename_ext      // 488
+// NYI mremap_encrypted  // 489
+#endif /* DARWIN_VERS >= DARWIN_10_10 */
 
 // Mach message helpers
 DECL_TEMPLATE(darwin, mach_port_set_context);
@@ -515,6 +592,8 @@ DECL_TEMPLATE(darwin, host_page_size);
 DECL_TEMPLATE(darwin, host_get_io_master);
 DECL_TEMPLATE(darwin, host_get_clock_service);
 DECL_TEMPLATE(darwin, host_request_notification);
+DECL_TEMPLATE(darwin, host_create_mach_voucher);
+DECL_TEMPLATE(darwin, host_get_special_port);
 DECL_TEMPLATE(darwin, mach_port_type);
 DECL_TEMPLATE(darwin, mach_port_extract_member);
 DECL_TEMPLATE(darwin, mach_port_allocate);
@@ -531,11 +610,15 @@ DECL_TEMPLATE(darwin, mach_port_get_attributes);
 DECL_TEMPLATE(darwin, mach_port_set_attributes);
 DECL_TEMPLATE(darwin, mach_port_insert_member);
 DECL_TEMPLATE(darwin, task_get_special_port);
+DECL_TEMPLATE(darwin, task_set_special_port);
 DECL_TEMPLATE(darwin, task_get_exception_ports);
 DECL_TEMPLATE(darwin, semaphore_create);
 DECL_TEMPLATE(darwin, semaphore_destroy);
 DECL_TEMPLATE(darwin, task_policy_set);
+DECL_TEMPLATE(darwin, mach_ports_register);
 DECL_TEMPLATE(darwin, mach_ports_lookup);
+DECL_TEMPLATE(darwin, task_info);
+DECL_TEMPLATE(darwin, task_set_info);
 DECL_TEMPLATE(darwin, task_threads);
 DECL_TEMPLATE(darwin, task_suspend);
 DECL_TEMPLATE(darwin, task_resume);
@@ -565,6 +648,7 @@ DECL_TEMPLATE(darwin, thread_terminate);
 DECL_TEMPLATE(darwin, thread_create);
 DECL_TEMPLATE(darwin, thread_create_running);
 DECL_TEMPLATE(darwin, thread_suspend);
+DECL_TEMPLATE(darwin, thread_resume);
 DECL_TEMPLATE(darwin, thread_get_state);
 DECL_TEMPLATE(darwin, thread_policy);
 DECL_TEMPLATE(darwin, thread_policy_set);
@@ -578,7 +662,7 @@ DECL_TEMPLATE(darwin, mach_msg_task);
 DECL_TEMPLATE(darwin, mach_msg_thread);
 
 // Mach traps
-#if DARWIN_VERS == DARWIN_10_8 || DARWIN_VERS == DARWIN_10_9
+#if DARWIN_VERS >= DARWIN_10_8
 DECL_TEMPLATE(darwin, kernelrpc_mach_vm_allocate_trap);
 DECL_TEMPLATE(darwin, kernelrpc_mach_vm_deallocate_trap);
 DECL_TEMPLATE(darwin, kernelrpc_mach_vm_protect_trap);
@@ -597,7 +681,7 @@ DECL_TEMPLATE(darwin, kernelrpc_mach_port_guard_trap);
 DECL_TEMPLATE(darwin, kernelrpc_mach_port_unguard_trap);
 DECL_TEMPLATE(darwin, iopolicysys);
 DECL_TEMPLATE(darwin, process_policy);
-#endif /* DARWIN_VERS == DARWIN_10_8 || DARWIN_VERS == DARWIN_10_9 */
+#endif /* DARWIN_VERS >= DARWIN_10_8 */
 DECL_TEMPLATE(darwin, mach_msg_unhandled);
 DECL_TEMPLATE(darwin, mach_msg_unhandled_check);
 DECL_TEMPLATE(darwin, mach_msg);
