@@ -526,10 +526,11 @@ DECL_TEMPLATE (mips_linux, sys_rt_sigreturn);
 DECL_TEMPLATE (mips_linux, sys_cacheflush);
 DECL_TEMPLATE (mips_linux, sys_set_thread_area);
 DECL_TEMPLATE (mips_linux, sys_pipe);
+DECL_TEMPLATE (mips_linux, sys_prctl);
 
 PRE(sys_mmap2) 
 {
-  /* Exactly like sys_mmap() except the file offset is specified in pagesize
+  /* Exactly like sys_mmap() except the file offset is specified in 4096 byte 
      units rather than bytes, so that it can be used for files bigger than
      2^32 bytes. */
   SysRes r;
@@ -539,7 +540,7 @@ PRE(sys_mmap2)
                 unsigned long, prot, unsigned long, flags,
                 unsigned long, fd, unsigned long, offset);
   r = mips_PRE_sys_mmap(tid, ARG1, ARG2, ARG3, ARG4, ARG5,
-                        VKI_PAGE_SIZE * (Off64T) ARG6);
+                        4096 * (Off64T) ARG6);
   SET_STATUS_from_SysRes(r);
 } 
 
@@ -784,6 +785,52 @@ POST(sys_pipe)
    }
 }
 
+PRE (sys_prctl)
+{
+   switch (ARG1) {
+      case VKI_PR_SET_FP_MODE:
+      {
+         VexArchInfo vai;
+         VG_(machine_get_VexArchInfo)(NULL, &vai);
+         /* Reject unsupported modes */
+         if ((ARG2 & ~VKI_PR_FP_MODE_FR) ||
+             ((ARG2 & VKI_PR_FP_MODE_FR) &&
+              !VEX_MIPS_HOST_FP_MODE(vai.hwcaps))) {
+            SET_STATUS_Failure(VKI_EOPNOTSUPP);
+         } else {
+            if (!(VG_(threads)[tid].arch.vex.guest_CP0_status &
+                  MIPS_CP0_STATUS_FR) != !(ARG2 & VKI_PR_FP_MODE_FR)) {
+               ThreadId t;
+               for (t = 1; t < VG_N_THREADS; t++) {
+                  if (VG_(threads)[t].status != VgTs_Empty) {
+                     if (ARG2 & VKI_PR_FP_MODE_FR) {
+                        VG_(threads)[t].arch.vex.guest_CP0_status |=
+                        MIPS_CP0_STATUS_FR;
+                     } else {
+                        VG_(threads)[t].arch.vex.guest_CP0_status &=
+                        ~MIPS_CP0_STATUS_FR;
+                     }
+                  }
+               }
+               /* Discard all translations */
+               VG_(discard_translations)(0, 0xfffffffful, "prctl(PR_SET_FP_MODE)");
+            }
+            SET_STATUS_Success(0);
+         }
+         break;
+      }
+      case VKI_PR_GET_FP_MODE:
+         if (VG_(threads)[tid].arch.vex.guest_CP0_status & MIPS_CP0_STATUS_FR)
+            SET_STATUS_Success(VKI_PR_FP_MODE_FR);
+         else
+            SET_STATUS_Success(0);
+         break;
+      default:
+         WRAPPER_PRE_NAME(linux, sys_prctl)(tid, layout, arrghs, status, flags);
+         break;
+   }
+}
+
 #undef PRE
 #undef POST
 
@@ -991,7 +1038,7 @@ static SyscallTableEntry syscall_main_table[] = {
    //..
    LINX_ (__NR_setresgid,              sys_setresgid),               // 190
    LINXY (__NR_getresgid,              sys_getresgid),               // 191
-   LINXY (__NR_prctl,                  sys_prctl),                   // 192
+   PLAX_ (__NR_prctl,                  sys_prctl),                   // 192
    PLAX_ (__NR_rt_sigreturn,           sys_rt_sigreturn),            // 193
    LINXY (__NR_rt_sigaction,           sys_rt_sigaction),            // 194
    LINXY (__NR_rt_sigprocmask,         sys_rt_sigprocmask),          // 195
@@ -1091,7 +1138,7 @@ static SyscallTableEntry syscall_main_table[] = {
    LINX_ (__NR_readlinkat,             sys_readlinkat),              // 298
    LINX_ (__NR_fchmodat,               sys_fchmodat),                // 299
    LINX_ (__NR_faccessat,              sys_faccessat),               // 300
-   //..
+   LINXY (__NR_pselect6,               sys_pselect6),                // 301
    LINXY (__NR_ppoll,                  sys_ppoll),                   // 302
    //..
    LINX_ (__NR_set_robust_list,        sys_set_robust_list),         // 309
