@@ -88,6 +88,7 @@ VgNeeds VG_(needs) = {
    .core_errors          = False,
    .tool_errors          = False,
    .libc_freeres         = False,
+   .cxx_freeres          = False,
    .superblock_discards  = False,
    .command_line_options = False,
    .client_requests      = False,
@@ -216,6 +217,7 @@ Bool VG_(sanity_check_needs)(const HChar** failmsg)
 
 // These ones don't require any tool-supplied functions
 NEEDS(libc_freeres)
+NEEDS(cxx_freeres)
 NEEDS(core_errors)
 NEEDS(var_info)
 
