@@ -7,7 +7,7 @@
    This file is part of Valgrind, a dynamic binary instrumentation
    framework.
 
-   Copyright (C) 2005-2012 Nicholas Nethercote
+   Copyright (C) 2005-2015 Nicholas Nethercote
       njn@valgrind.org
 
    This program is free software; you can redistribute it and/or
@@ -31,6 +31,8 @@
 #ifndef __PUB_TOOL_OSET_H
 #define __PUB_TOOL_OSET_H
 
+#include "pub_tool_basics.h"   // Word
+
 // This module implements an ordered set, a data structure with fast
 // (eg. amortised log(n) or better) insertion, lookup and deletion of
 // elements.  It does not allow duplicates, and will assert if you insert a
@@ -54,9 +56,12 @@
 //   function even a (non-overlapping) interval list can be created.  But
 //   the cost of calling a function for every comparison can be high during
 //   lookup.  If no comparison function is provided, we assume that keys are
-//   (signed or unsigned) words, and that the key is the first word in each
+//   unsigned words, and that the key is the first word in each
 //   element.  This fast comparison is suitable for an OSet containing
 //   structs where the first element is an Addr, for example.
+//   Do not assume fast comparison works properly with signed words.
+//   A.o. iterating over the values will not return them in the correct
+//   order.
 //
 // Each OSet interface also has an iterator, which makes it simple to
 // traverse all the nodes in order.  Note that the iterator maintains state
@@ -77,31 +82,30 @@ typedef struct _OSet     OSet;
 // - Free:  frees a chunk of memory allocated with Alloc.
 
 typedef Word  (*OSetCmp_t)         ( const void* key, const void* elem );
-typedef void* (*OSetAlloc_t)       ( HChar* cc, SizeT szB );
+typedef void* (*OSetAlloc_t)       ( const HChar* cc, SizeT szB );
 typedef void  (*OSetFree_t)        ( void* p );
 
 /*--------------------------------------------------------------------*/
 /*--- Creating and destroying OSets (UWord)                        ---*/
 /*--------------------------------------------------------------------*/
 
-// * Create: allocates and initialises the OSet.  Arguments:
-//   - alloc     The allocation function used internally for allocating the
-//               OSet and all its nodes.
+// * Create: allocates and initialises the OSet.  Never returns NULL.
+//   Parameters:
+//   - alloc_fn  The allocation function used internally for allocating the
+//               OSet and all its nodes. It must not return NULL (that is,
+//               if it returns it must have succeeded.)
 //   - cc        Cost centre string used by 'alloc'.
-//   - free      The deallocation function used internally for freeing nodes
+//   - free_fn   The deallocation function used internally for freeing nodes
 //               called by VG_(OSetWord_Destroy)().
 //
-// * CreateWithCmp: like Create, but you specify your own comparison
-//   function.
-//
 // * Destroy: frees all nodes in the table, plus the memory used by
 //   the table itself.  The passed-in function is called on each node first
 //   to allow the destruction of any attached resources;  if NULL it is not
 //   called.
 
-extern OSet* VG_(OSetWord_Create)       ( OSetAlloc_t alloc, HChar* cc, 
-                                          OSetFree_t _free );
-extern void  VG_(OSetWord_Destroy)      ( OSet* os );
+extern OSet* VG_(OSetWord_Create) ( OSetAlloc_t alloc_fn, const HChar* cc, 
+                                    OSetFree_t free_fn );
+extern void  VG_(OSetWord_Destroy) ( OSet* os );
 
 /*--------------------------------------------------------------------*/
 /*--- Operations on OSets (UWord)                                  ---*/
@@ -143,9 +147,9 @@ extern void  VG_(OSetWord_Destroy)      ( OSet* os );
 //   they will return False if VG_(OSetWord_Next)() is called without an
 //   intervening call to VG_(OSetWord_ResetIter)().
 
-extern Word  VG_(OSetWord_Size)         ( OSet* os );
+extern Word  VG_(OSetWord_Size)         ( const OSet* os );
 extern void  VG_(OSetWord_Insert)       ( OSet* os, UWord val );
-extern Bool  VG_(OSetWord_Contains)     ( OSet* os, UWord val );
+extern Bool  VG_(OSetWord_Contains)     ( const OSet* os, UWord val );
 extern Bool  VG_(OSetWord_Remove)       ( OSet* os, UWord val );
 extern void  VG_(OSetWord_ResetIter)    ( OSet* os );
 extern Bool  VG_(OSetWord_Next)         ( OSet* os, /*OUT*/UWord* val );
@@ -155,17 +159,20 @@ extern Bool  VG_(OSetWord_Next)         ( OSet* os, /*OUT*/UWord* val );
 /*--- Creating and destroying OSets and OSet members (Gen)         ---*/
 /*--------------------------------------------------------------------*/
 
-// * Create: allocates and initialises the OSet.  Arguments:
+// * Create: allocates and initialises the OSet. Never returns NULL.
+//   Parameters:
 //   - keyOff    The offset of the key within the element.
 //   - cmp       The comparison function between keys and elements, or NULL
 //               if the OSet should use fast comparisons.
-//   - alloc     The allocation function used for allocating the OSet itself;
+//   - alloc_fn  The allocation function used for allocating the OSet itself;
+//               It must not return NULL (that is, if it returns it must
+//               have succeeded.)
 //               If a pool allocator is used, it's called to allocate pool of
 //               nodes.
 //               If no pool allocator is used, it's called for each
 //               invocation of VG_(OSetGen_AllocNode)().
 //   - cc        Cost centre string used by 'alloc'.
-//   - free      If no pool allocator is used, this is the deallocation
+//   - free_fn   If no pool allocator is used, this is the deallocation
 //               function used by VG_(OSetGen_FreeNode)() and
 //               VG_(OSetGen_Destroy)().
 //               If a pool allocator is used, the memory used by the nodes is
@@ -197,13 +204,14 @@ extern Bool  VG_(OSetWord_Next)         ( OSet* os, /*OUT*/UWord* val );
 //   lead to assertions in Valgrind's allocator.
 
 extern OSet* VG_(OSetGen_Create)    ( PtrdiffT keyOff, OSetCmp_t cmp,
-                                      OSetAlloc_t alloc, HChar* cc,
-                                      OSetFree_t _free);
+                                      OSetAlloc_t alloc_fn, const HChar* cc,
+                                      OSetFree_t free_fn);
 
 
 extern OSet* VG_(OSetGen_Create_With_Pool)    ( PtrdiffT keyOff, OSetCmp_t cmp,
-                                                OSetAlloc_t alloc, HChar* cc,
-                                                OSetFree_t _free,
+                                                OSetAlloc_t alloc_fn,
+                                                const HChar* cc,
+                                                OSetFree_t free_fn,
                                                 SizeT poolSize,
                                                 SizeT maxEltSize);
 // Same as VG_(OSetGen_Create) but created OSet will use a pool allocator to
@@ -217,22 +225,22 @@ extern OSet* VG_(OSetGen_Create_With_Pool)    ( PtrdiffT keyOff, OSetCmp_t cmp,
 // always <= than a (small) maximum value.
 // In such a case, allocating the nodes in pools reduces significantly
 // the memory overhead needed by each node.
-// When a node is freed (i.e. OsetGen_Freenode is called), the node is
+// When a node is freed (i.e. OSetGen_Freenode is called), the node is
 // put back in the pool allocator free list (for sub-sequent re-use by
-// Osetgen_Allocnode). Note that the pool memory is only released when
+// OSetGen_AllocNode). Note that the pool memory is only released when
 // the pool is destroyed : calls to VG_(OSetGen_Free) do not cause
-// any calls to OsetFree_t _free function.
+// any calls to OSetFree_t _free function.
 // If there are several OSet managing similar such elements, it might be
 // interesting to use a shared pool for these OSet.
 // To have multiple OSets sharing a pool allocator, create the first OSet
-// with VG_(OSetGen_Create). Create subsequent OSet with
+// with VG_(OSetGen_Create_With_Pool). Create subsequent OSet with
 // VG_(OSetGen_EmptyClone).
 
 extern void  VG_(OSetGen_Destroy)   ( OSet* os );
-extern void* VG_(OSetGen_AllocNode) ( OSet* os, SizeT elemSize );
-extern void  VG_(OSetGen_FreeNode)  ( OSet* os, void* elem );
+extern void* VG_(OSetGen_AllocNode) ( const OSet* os, SizeT elemSize );
+extern void  VG_(OSetGen_FreeNode)  ( const OSet* os, void* elem );
 
-extern OSet* VG_(OSetGen_EmptyClone) (OSet* os);
+extern OSet* VG_(OSetGen_EmptyClone) (const OSet* os);
 // Creates a new empty OSet.
 // The new OSet will have the same characteristics as os.
 // If os uses a pool allocator, this pool allocator will be shared with
@@ -290,7 +298,7 @@ extern OSet* VG_(OSetGen_EmptyClone) (OSet* os);
 //   they will return NULL if VG_(OSetGen_Next)() is called without an
 //   intervening call to VG_(OSetGen_ResetIter)().
 
-extern Word  VG_(OSetGen_Size)         ( const OSet* os );
+extern UInt  VG_(OSetGen_Size)         ( const OSet* os );
 extern void  VG_(OSetGen_Insert)       ( OSet* os, void* elem );
 extern Bool  VG_(OSetGen_Contains)     ( const OSet* os, const void* key );
 extern void* VG_(OSetGen_Lookup)       ( const OSet* os, const void* key );
