@@ -746,13 +746,20 @@ void MC_(record_address_error) ( ThreadId tid, Addr a, Int szB,
    if (VG_(is_watched)( (isWrite ? write_watchpoint : read_watchpoint), a, szB))
       return;
 
-   just_below_esp = is_just_below_ESP( VG_(get_SP)(tid), a );
+   Addr current_sp = VG_(get_SP)(tid);
+   just_below_esp = is_just_below_ESP( current_sp, a );
 
    /* If this is caused by an access immediately below %ESP, and the
       user asks nicely, we just ignore it. */
    if (MC_(clo_workaround_gcc296_bugs) && just_below_esp)
       return;
 
+   /* Also, if this is caused by an access in the range of offsets
+      below the stack pointer as described by
+      --ignore-range-below-sp, ignore it. */
+   if (MC_(in_ignored_range_below_sp)( current_sp, a, szB ))
+      return;
+
    extra.Err.Addr.isWrite   = isWrite;
    extra.Err.Addr.szB       = szB;
    extra.Err.Addr.maybe_gcc = just_below_esp;
@@ -925,6 +932,30 @@ void MC_(record_user_error) ( ThreadId tid, Addr a,
    VG_(maybe_record_error)( tid, Err_User, a, /*s*/NULL, &extra );
 }
 
+Bool MC_(is_mempool_block)(MC_Chunk* mc_search)
+{
+   MC_Mempool* mp;
+
+   if (!MC_(mempool_list))
+      return False;
+
+   // A chunk can only come from a mempool if a custom allocator
+   // is used. No search required for other kinds.
+   if (mc_search->allockind == MC_AllocCustom) {
+      VG_(HT_ResetIter)( MC_(mempool_list) );
+      while ( (mp = VG_(HT_Next)(MC_(mempool_list))) ) {
+         MC_Chunk* mc;
+         VG_(HT_ResetIter)(mp->chunks);
+         while ( (mc = VG_(HT_Next)(mp->chunks)) ) {
+            if (mc == mc_search)
+               return True;
+         }
+      }
+   }
+
+   return False;
+}
+ 
 /*------------------------------------------------------------*/
 /*--- Other error operations                               ---*/
 /*------------------------------------------------------------*/
@@ -1016,7 +1047,8 @@ Bool addr_is_in_MC_Chunk_with_REDZONE_SZB(MC_Chunk* mc, Addr a, SizeT rzB)
 
 // Forward declarations
 static Bool client_block_maybe_describe( Addr a, AddrInfo* ai );
-static Bool mempool_block_maybe_describe( Addr a, AddrInfo* ai );
+static Bool mempool_block_maybe_describe( Addr a, Bool is_metapool,
+                                          AddrInfo* ai );
 
 
 /* Describe an address as best you can, for error messages,
@@ -1031,10 +1063,12 @@ static void describe_addr ( Addr a, /*OUT*/AddrInfo* ai )
    if (client_block_maybe_describe( a, ai )) {
       return;
    }
-   /* -- Perhaps it's in mempool block? -- */
-   if (mempool_block_maybe_describe( a, ai )) {
+
+   /* -- Perhaps it's in mempool block (non-meta)? -- */
+   if (mempool_block_maybe_describe( a, /*is_metapool*/ False, ai)) {
       return;
    }
+
    /* Blocks allocated by memcheck malloc functions are either
       on the recently freed list or on the malloc-ed list.
       Custom blocks can be on both : a recently freed block might
@@ -1046,7 +1080,8 @@ static void describe_addr ( Addr a, /*OUT*/AddrInfo* ai )
    /* -- Search for a currently malloc'd block which might bracket it. -- */
    VG_(HT_ResetIter)(MC_(malloc_list));
    while ( (mc = VG_(HT_Next)(MC_(malloc_list))) ) {
-      if (addr_is_in_MC_Chunk_default_REDZONE_SZB(mc, a)) {
+      if (!MC_(is_mempool_block)(mc) && 
+           addr_is_in_MC_Chunk_default_REDZONE_SZB(mc, a)) {
          ai->tag = Addr_Block;
          ai->Addr.Block.block_kind = Block_Mallocd;
          if (MC_(get_freed_block_bracketting)( a ))
@@ -1063,7 +1098,7 @@ static void describe_addr ( Addr a, /*OUT*/AddrInfo* ai )
    }
    /* -- Search for a recently freed block which might bracket it. -- */
    mc = MC_(get_freed_block_bracketting)( a );
-   if (mc) {
+   if (mc && !MC_(is_mempool_block)(mc)) {
       ai->tag = Addr_Block;
       ai->Addr.Block.block_kind = Block_Freed;
       ai->Addr.Block.block_desc = "block";
@@ -1075,6 +1110,16 @@ static void describe_addr ( Addr a, /*OUT*/AddrInfo* ai )
       return;
    }
 
+   /* -- Perhaps it's in a meta mempool block? -- */
+   /* This test is done last, because metapool blocks overlap with blocks
+      handed out to the application. That makes every heap address part of
+      a metapool block, so the interesting cases are handled first.
+      This final search is a last-ditch attempt. When found, it is probably
+      an error in the custom allocator itself. */
+   if (mempool_block_maybe_describe( a, /*is_metapool*/ True, ai )) {
+      return;
+   }
+
    /* No block found. Search a non-heap block description. */
    VG_(describe_addr) (a, ai);
 }
@@ -1215,7 +1260,7 @@ static Bool client_block_maybe_describe( Addr a,
 }
 
 
-static Bool mempool_block_maybe_describe( Addr a,
+static Bool mempool_block_maybe_describe( Addr a, Bool is_metapool,
                                           /*OUT*/AddrInfo* ai )
 {
    MC_Mempool* mp;
@@ -1223,7 +1268,7 @@ static Bool mempool_block_maybe_describe( Addr a,
 
    VG_(HT_ResetIter)( MC_(mempool_list) );
    while ( (mp = VG_(HT_Next)(MC_(mempool_list))) ) {
-      if (mp->chunks != NULL) {
+      if (mp->chunks != NULL && mp->metapool == is_metapool) {
          MC_Chunk* mc;
          VG_(HT_ResetIter)(mp->chunks);
          while ( (mc = VG_(HT_Next)(mp->chunks)) ) {
@@ -1349,7 +1394,7 @@ Bool MC_(read_extra_suppression_info) ( Int fd, HChar** bufpp,
       if (eof) return True; // old LeakSupp style, no match-leak-kinds line.
       if (0 == VG_(strncmp)(*bufpp, "match-leak-kinds:", 17)) {
          i = 17;
-         while ((*bufpp)[i] && VG_(isspace((*bufpp)[i])))
+         while ((*bufpp)[i] && VG_(isspace)((*bufpp)[i]))
             i++;
          if (!VG_(parse_enum_set)(MC_(parse_leak_kinds_tokens),
                                   True/*allow_all*/,
@@ -1394,7 +1439,7 @@ Bool MC_(error_matches_suppression) ( const Error* err, const Supp* su )
 {
    Int       su_szB;
    MC_Error* extra = VG_(get_error_extra)(err);
-   ErrorKind ekind = VG_(get_error_kind )(err);
+   ErrorKind ekind = VG_(get_error_kind)(err);
 
    switch (VG_(get_supp_kind)(su)) {
       case ParamSupp:
@@ -1522,7 +1567,7 @@ const HChar* MC_(get_error_name) ( const Error* err )
 SizeT MC_(get_extra_suppression_info) ( const Error* err,
                                         /*OUT*/HChar* buf, Int nBuf )
 {
-   ErrorKind ekind = VG_(get_error_kind )(err);
+   ErrorKind ekind = VG_(get_error_kind)(err);
    tl_assert(buf);
    tl_assert(nBuf >= 1);
 
