@@ -93,6 +93,9 @@ typedef
       Addr          pool;           // pool identifier
       SizeT         rzB;            // pool red-zone size
       Bool          is_zeroed;      // allocations from this pool are zeroed
+      Bool          auto_free;      // De-alloc block frees all chunks in block
+      Bool          metapool;       // These chunks are VALGRIND_MALLOC_LIKE
+                                    // memory, and used as pool.
       VgHashTable  *chunks;         // chunks associated with this pool
    }
    MC_Mempool;
@@ -105,7 +108,8 @@ void* MC_(new_block)  ( ThreadId tid,
 void MC_(handle_free) ( ThreadId tid,
                         Addr p, UInt rzB, MC_AllocKind kind );
 
-void MC_(create_mempool)  ( Addr pool, UInt rzB, Bool is_zeroed );
+void MC_(create_mempool)  ( Addr pool, UInt rzB, Bool is_zeroed,
+                            Bool auto_free, Bool metapool );
 void MC_(destroy_mempool) ( Addr pool );
 void MC_(mempool_alloc)   ( ThreadId tid, Addr pool,
                             Addr addr, SizeT size );
@@ -114,6 +118,7 @@ void MC_(mempool_trim)    ( Addr pool, Addr addr, SizeT size );
 void MC_(move_mempool)    ( Addr poolA, Addr poolB );
 void MC_(mempool_change)  ( Addr pool, Addr addrA, Addr addrB, SizeT size );
 Bool MC_(mempool_exists)  ( Addr pool );
+Bool MC_(is_mempool_block)( MC_Chunk* mc_search );
 
 /* Searches for a recently freed block which might bracket Addr a.
    Return the MC_Chunk* for this block or NULL if no bracketting block
@@ -316,6 +321,12 @@ enum {
    MCPE_DIE_MEM_STACK_128,
    MCPE_DIE_MEM_STACK_144,
    MCPE_DIE_MEM_STACK_160,
+   MCPE_MAKE_STACK_UNINIT_W_O,
+   MCPE_MAKE_STACK_UNINIT_NO_O,
+   MCPE_MAKE_STACK_UNINIT_128_NO_O,
+   MCPE_MAKE_STACK_UNINIT_128_NO_O_ALIGNED_16,
+   MCPE_MAKE_STACK_UNINIT_128_NO_O_ALIGNED_8,
+   MCPE_MAKE_STACK_UNINIT_128_NO_O_SLOWCASE,
    /* Do not add enumerators past this line. */
    MCPE_LAST
 };
@@ -561,6 +572,10 @@ void MC_(pp_describe_addr) (Addr a);
 /* Is this address in a user-specified "ignored range" ? */
 Bool MC_(in_ignored_range) ( Addr a );
 
+/* Is this address in a user-specified "ignored range of offsets below
+   the current thread's stack pointer?" */
+Bool MC_(in_ignored_range_below_sp) ( Addr sp, Addr a, UInt szB );
+
 
 /*------------------------------------------------------------*/
 /*--- Client blocks                                        ---*/
@@ -704,6 +719,12 @@ extern Bool MC_(clo_show_mismatched_frees);
    operations? Default: NO */
 extern Bool MC_(clo_expensive_definedness_checks);
 
+/* Do we have a range of stack offsets to ignore?  Default: NO */
+extern Bool MC_(clo_ignore_range_below_sp);
+extern UInt MC_(clo_ignore_range_below_sp__first_offset);
+extern UInt MC_(clo_ignore_range_below_sp__last_offset);
+
+
 /*------------------------------------------------------------*/
 /*--- Instrumentation                                      ---*/
 /*------------------------------------------------------------*/
@@ -749,8 +770,14 @@ VG_REGPARM(1) UWord MC_(helperc_LOADV16be)  ( Addr );
 VG_REGPARM(1) UWord MC_(helperc_LOADV16le)  ( Addr );
 VG_REGPARM(1) UWord MC_(helperc_LOADV8)     ( Addr );
 
-void MC_(helperc_MAKE_STACK_UNINIT) ( Addr base, UWord len,
-                                                 Addr nia );
+VG_REGPARM(3)
+void MC_(helperc_MAKE_STACK_UNINIT_w_o) ( Addr base, UWord len, Addr nia );
+
+VG_REGPARM(2)
+void MC_(helperc_MAKE_STACK_UNINIT_no_o) ( Addr base, UWord len );
+
+VG_REGPARM(1)
+void MC_(helperc_MAKE_STACK_UNINIT_128_no_o) ( Addr base );
 
 /* Origin tag load/store helpers */
 VG_REGPARM(2) void  MC_(helperc_b_store1) ( Addr a, UWord d32 );
@@ -776,6 +803,9 @@ IRSB* MC_(instrument) ( VgCallbackClosure* closure,
 
 IRSB* MC_(final_tidy) ( IRSB* );
 
+/* Check some assertions to do with the instrumentation machinery. */
+void MC_(do_instrumentation_startup_checks)( void );
+
 #endif /* ndef __MC_INCLUDE_H */
 
 /*--------------------------------------------------------------------*/
