@@ -9,7 +9,7 @@ int main (int argc, char **argv)
 
    CLOSE_INHERITED_FDS;
 
-   sprintf(filename, "/tmp/file.%d", getpid());
+   sprintf(filename, "/tmp/file.%ld", (long) getpid());
    (void) DO( creat(filename, 0) );
    (void) DO( unlink(filename) );
    return 0;
